﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Nutshell.Test.Support;
using System.Linq;

namespace Nutshell.ThreadWorkerPool.Test
{
    [TestClass]
    public class ThreadWorkerPoolManagerTest
    {
        [TestMethod]
        public void TestCreateNewPool()
        {
            using (var manager = ThreadWorkerPoolManager.Instance)
            {
                ArgumentExceptionHelper
                    .CheckArgumentNullException("poolName", () => manager.CreateNewPool("", null, ThreadWorkerPoolSettings.Default));

                ArgumentExceptionHelper
                    .CheckArgumentNullException("workingHandler", () => manager.CreateNewPool("TestPool", null, ThreadWorkerPoolSettings.Default));

                var pool1 = manager.CreateNewPool("TestPool", () => { }, ThreadWorkerPoolSettings.Default);

                ArgumentExceptionHelper.CheckArgumentException("poolName", () =>
                {
                    var pool2 = manager.CreateNewPool("TestPool", () => { }, ThreadWorkerPoolSettings.Default);
                });

                var allPools = manager.GetAll();
                Assert.AreEqual(1, allPools.Count());
                Assert.AreEqual(pool1, allPools.FirstOrDefault());
            }
        }

        [TestMethod]
        public void TestGetPool()
        {
            using (var manager = ThreadWorkerPoolManager.Instance)
            {
                manager.CreateNewPool("TestPool", () => { }, ThreadWorkerPoolSettings.Default);

                ArgumentExceptionHelper
                    .CheckArgumentNullException("poolName", () => manager.Get(""));

                var pool = manager.Get("TestPool2");
                Assert.IsNull(pool);

                pool = manager.Get("TestPool");
                Assert.IsNotNull(pool);
            }
        }

        [TestMethod]
        public void TestDestory()
        {
            using (var manager = ThreadWorkerPoolManager.Instance)
            {
                var pool = manager.CreateNewPool("TestPool", () => { }, ThreadWorkerPoolSettings.Default);

                manager.Destory(pool as ThreadWorkerPoolBase);
                var allPools = manager.GetAll();
                Assert.AreEqual(0, allPools.Count());
            }
        }
    }
}
