﻿using Nutshell.ThreadWorkerPool.Handlers;

namespace Nutshell.ThreadWorkerPool
{
    public sealed class ThreadWorkerPool : ThreadWorkerPoolBase, IThreadWorkerPool
    {
        private readonly WorkingHandler _workingHandler;

        internal ThreadWorkerPool(string name, WorkingHandler workingHandler, ThreadWorkerPoolSettings settings)
            : base(name, settings)
        {
            _workingHandler = workingHandler;
        }

        protected override ThreadWorkerPoolItem CreatePoolItem(int index, int idleTime)
        {
            string workerName = _workingHandler.ToString() + "_" + index.ToString();
            var settings = base.Settings;
            settings.IdleTime = idleTime;
            var worker = new ThreadWorker(workerName, _workingHandler, settings.Priority, settings.IsBackground, settings.IdleTime);
            return new ThreadWorkerPoolItem(this, worker, settings);
        }

        public IThreadWorker Take()
        {
            IThreadWorker worker;
            this.TryTake(-1, out worker);
            return worker;
        }

        public bool TryTake(out IThreadWorker threadWorker)
        {
            return this.TryTake(0, out threadWorker);
        }

        public bool TryTake(int timeout, out IThreadWorker threadWorker)
        {
            ThreadWorkerBase worker;
            if (base.TryTake(timeout, out worker))
            {
                threadWorker = (IThreadWorker)worker;
                return true;
            }
            threadWorker = null;
            return false;
        }
    }

    public sealed class ThreadWorkerPool<T> : ThreadWorkerPoolBase, IThreadWorkerPool<T>
    {
        private readonly WorkingHandler<T> _workingHandler;

        internal ThreadWorkerPool(string name, WorkingHandler<T> workingHandler, ThreadWorkerPoolSettings settings)
            : base(name, settings)
        {
            _workingHandler = workingHandler;
        }

        protected override ThreadWorkerPoolItem CreatePoolItem(int index, int idleTime)
        {
            string workerName = _workingHandler.ToString() + "_" + index.ToString();
            var settings = base.Settings;
            settings.IdleTime = idleTime;
            var worker = new ThreadWorker<T>(workerName, _workingHandler, settings.Priority, settings.IsBackground, settings.IdleTime);
            return new ThreadWorkerPoolItem(this, worker, settings);
        }

        public IThreadWorker<T> Take()
        {
            IThreadWorker<T> worker;
            this.TryTake(-1, out worker);
            return worker;
        }

        public bool TryTake(out IThreadWorker<T> threadWorker)
        {
            return this.TryTake(0, out threadWorker);
        }

        public bool TryTake(int timeout, out IThreadWorker<T> threadWorker)
        {
            ThreadWorkerBase worker;
            if (base.TryTake(timeout, out worker))
            {
                threadWorker = (IThreadWorker<T>)worker;
                return true;
            }
            threadWorker = null;
            return false;
        }
    }
}
